@@ -49,6 +49,8 @@ static void gnome_href_set_property	(GObject *object,
 					 const GValue * value,
 					 GParamSpec * pspec);
 static void gnome_href_realize		(GtkWidget *widget);
+static void gnome_href_style_set        (GtkWidget *widget,
+					 GtkStyle *previous_style);
 static void drag_data_get    		(GnomeHRef          *href,
 					 GdkDragContext     *context,
 					 GtkSelectionData   *selection_data,
@@ -109,6 +111,7 @@ gnome_href_class_init (GnomeHRefClass *klass)
 	gobject_class->get_property = gnome_href_get_property;
 
 	widget_class->realize = gnome_href_realize;
+	widget_class->style_set = gnome_href_style_set;
 	button_class->clicked = gnome_href_clicked;
 
 	/* By default we link to The World Food Programme */
@@ -130,9 +133,9 @@ gnome_href_class_init (GnomeHRefClass *klass)
 							       G_PARAM_WRITABLE)));
 
 	gtk_widget_class_install_style_property (GTK_WIDGET_CLASS (gobject_class),
-						 g_param_spec_boxed ("link_colour",
-								     _("Link colour"),
-								     _("Colour used to draw the link"),
+						 g_param_spec_boxed ("link_color",
+								     _("Link color"),
+								     _("Color used to draw the link"),
 								     GDK_TYPE_COLOR,
 								     G_PARAM_READABLE));
 }
@@ -140,27 +143,11 @@ gnome_href_class_init (GnomeHRefClass *klass)
 static void
 gnome_href_instance_init (GnomeHRef *href)
 {
-        GdkColor *link_colour;
-	GdkColor blue = { 0, 0x0000, 0x0000, 0xffff };
-
 	href->_priv = g_new0(GnomeHRefPrivate, 1);
 
 	href->_priv->label = gtk_label_new("");
 	gtk_widget_ref(href->_priv->label);
 
-	gtk_widget_style_get (GTK_WIDGET(href),
-			      "link_colour", &link_colour,
-			      NULL);
-	if (!link_colour)
-		link_colour = &blue;
-	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
-			      GTK_STATE_NORMAL, link_colour);
-	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
-			      GTK_STATE_ACTIVE, link_colour);
-	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
-			      GTK_STATE_PRELIGHT, link_colour);
-	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
-			      GTK_STATE_SELECTED, link_colour);
 
 	gtk_button_set_relief(GTK_BUTTON(href), GTK_RELIEF_NONE);
 
@@ -391,14 +378,15 @@ gnome_href_clicked (GtkButton *button)
 
   /* FIXME: Use the error variable from gnome_url_show */
   if(!gnome_url_show(href->_priv->url, NULL)) {
-      GtkWidget *dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR,
-						 GTK_BUTTONS_OK,
-						 _("Error occured while trying to launch the "
-						   "URL handler.\n"
-						   "Please check the settings in the "
-						   "Control Center if they are correct."));
-
-      gtk_dialog_run(GTK_DIALOG(dialog));
+    GtkWidget *dialog = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_ERROR,
+					       GTK_BUTTONS_OK,
+					       _("Error occured while trying to launch the "
+						 "URL handler.\n"
+						 "Please check the settings in the "
+						 "Control Center if they are correct."));
+    
+    gtk_dialog_run(GTK_DIALOG(dialog));
+    gtk_widget_destroy(dialog);
   }
 }
 
@@ -453,6 +441,36 @@ gnome_href_realize(GtkWidget *widget)
 	gdk_cursor_unref (cursor);
 }
 
+static void
+gnome_href_style_set (GtkWidget *widget,
+		      GtkStyle *previous_style)
+{
+	GdkColor *link_color;
+	GdkColor blue = { 0, 0x0000, 0x0000, 0xffff };
+	GnomeHRef *href;
+
+	href = GNOME_HREF (widget);
+	
+	gtk_widget_style_get (widget,
+			      "link_color", &link_color,
+			      NULL);
+
+	if (!link_color)
+		link_color = &blue;
+
+	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
+			      GTK_STATE_NORMAL, link_color);
+	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
+			      GTK_STATE_ACTIVE, link_color);
+	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
+			      GTK_STATE_PRELIGHT, link_color);
+	gtk_widget_modify_fg (GTK_WIDGET(href->_priv->label),
+			      GTK_STATE_SELECTED, link_color);
+
+	if (link_color != &blue)
+		gdk_color_free (link_color);
+}
+
 static void
 gnome_href_set_property (GObject *object,
 			 guint param_id,
